@@ -6,6 +6,7 @@ from multiprocessing import Process, Manager, cpu_count
 import os
 from copy import deepcopy
 from scipy.interpolate import NearestNDInterpolator
+from scipy.ndimage import gaussian_filter
 import quadpy as qp
 import warnings
 import inspect
@@ -18,7 +19,7 @@ from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
 from reportlab.lib.units import inch
 
 
-__version__ = "1.0.12"
+__version__ = "1.0.14"
 
 class qctma(object):
     """
@@ -27,7 +28,7 @@ class qctma(object):
     """
 
     def __init__(self, dcm_path="", mesh_path="", gl2density=lambda x: x, density2E=lambda x: x, deltaE=0,
-                 process=False, save_mesh_path="", nb_process=1):
+                 process=False, save_mesh_path="", nb_process=1, gaussian_smoothing=0):
         """
         Constructs a qctma object from the Dicoms directory path, the original mesh file path, the gray level to density
         relation, the density to Young's modulus relation, and the Young's modulus delta step.
@@ -40,6 +41,7 @@ class qctma(object):
         :param process: If True, will process the data when instance is created.
         :param save_mesh_path: Path to the desired location of the final mesh.
         :param nb_process: Number of used process. if > 1, multiprocessing package will be used.
+        :param gaussian_smoothing: Sigma of the gaussian filter smoothing for the dicoms (0 means no gaussian smoothing)
         """
         self.dcm_path = dcm_path  # Path to the directory containing the Dicom files
         self.mesh_path = mesh_path  # Path to the mesh file
@@ -69,6 +71,9 @@ class qctma(object):
         self.interpolator = None  # Interpolator Young's modulus = f(x, y, z)
         self.e_elems = []  # 1D array of Young's modulus of each element
 
+        # DICOM PROCESSING
+        self.gaussian_smoothing = gaussian_smoothing
+
         self.nb_process = nb_process  # Number of core for the parallelized process
         if self.nb_process > 1:
             warnings.warn("WARNING: More than 1 process will be used. Check if your code runs inside <if __name__ == '__main__':> statement !")
@@ -147,6 +152,10 @@ class qctma(object):
             self.image_mat[i] = d.pixel_array * d.RescaleSlope + d.RescaleIntercept
             self.positions_z[i] = float(ds[i].ImagePositionPatient[2])
 
+        # Smoothing the Dicoms
+        if self.gaussian_smoothing:
+            self.image_mat = gaussian_filter(self.image_mat, self.gaussian_smoothing)
+
         print("Volume created.\n")
 
     def convert2E(self):
@@ -313,8 +322,8 @@ class qctma(object):
                                                [np.min((v1[-1][2], v3[-1][2])), np.max((v1[-1][2], v3[-1][2]))])
                 v_cube.append(deepcopy(cube_point))
 
-            if not elem_arg % 100:
-                print("id %i:" % id, elem_arg, time.time() - t0)
+            if (not elem_arg % 100) and id == 0:
+                print("\rid %i:" % id, elem_arg, time.time() - t0, end="")
         # Integrating the Young's modulus over each element and dividing it by the volume of each element to get the mean.
         # Note: the function used to get the volume is x / x and not 1 in order to have the right shape in the end.
         if is_cube:
@@ -324,6 +333,8 @@ class qctma(object):
             elems_mat = scheme.integrate(f, (v1, v2, v3, v4)) / scheme.integrate(lambda x: x[0] / x[0],
                                                                                  (v1, v2, v3, v4))
 
+        if id == 0:
+            print("id %i:" % id, "DONE", time.time() - t0)
         save_list[id] = elems_mat  # Saving the results in the save_list param
 
     def reduce_material_nb(self):
@@ -362,21 +373,11 @@ class qctma(object):
                 x = np.linspace(*first_guess, nb_sample)
                 f_x = f(x)
                 diff = np.abs(f_x - y)
-                argmin = np.argmin(diff)
+                argmin = np.nanargmin(diff)
                 if delta <= max_err:
                     break
-                if argmin == len(x) - 1:
-                    first_guess[0] = 0
-                    if first_guess[1] == 0:
-                        first_guess[1] = 1
-                    first_guess[1] *= 2
-                elif argmin == 0:
-                    first_guess[1] = 0
-                    if first_guess[0] == 0:
-                        first_guess[0] = -1
-                    first_guess[0] *= 2
-                else:
-                    first_guess = [x[argmin] - delta, x[argmin] + delta]
+                # TODO: TAKE CARE OF VALUES THAT ARE OUTSIDE INIT_GUESS
+                first_guess = [x[argmin] - delta, x[argmin] + delta]
             inv_values.append(x[argmin])
 
         return inv_values
